import { ItemStack } from '@minecraft/server'

class HopperTools {
    // NEARBY HOPPERS
    static facingInward(block, dir) {
        const facingDirectionKey = ["down", null, "north", "south", "west", "east"];
        const facingInwardDict = {
            "down": "above",
            "north": "south",
            "south": "north",
            "east": "west",
            "west": "east"
        }

        const facingDirNum = block.permutation.getState("facing_direction");
        return facingDirectionKey[facingDirNum] === facingInwardDict[dir];
    }

    static getInputs(block, itemName = null) {
        const dirs = ["north", "east", "south", "west", "above"];

        return dirs.map(dir => ({ dir, block: block[dir]() }))
            .filter(({ block, dir }) =>
                block.typeId === "minecraft:hopper" &&
                HopperTools.facingInward(block, dir) &&
                (itemName === false || HopperTools.hasItems(block, itemName))
            )
            .map(data => data.block);
    }

    static getOutput(block) {
        return block.below().typeId === 'minecraft:hopper' ? block.below() : null;
    }

    // INVENTORY INTERACTION
    static hasItems(hopper, itemName) {
        const inv = hopper.getComponent("inventory").container;

        for (let i = 0; i < inv.size; i++) {
            const item = inv.getItem(i);
            if (item === undefined) continue;
            if (item.typeId === itemName) return true;
        }

        return false;
    }

    static getContainer(block) {
        return block.getComponent("inventory").container;
    }

    static takeItem(hopper, itemName) {
        const inv = HopperTools.getContainer(hopper);

        for (let i = 0; i < inv.size; i++) {
            const item = inv.getItem(i);
            if (item === undefined || item.typeId != itemName) continue;

            if (item.amount === 1) {
                inv.setItem(i)
            } else {
                item.amount--;
                inv.setItem(i, item);
            }

            return;
        }
    }

    static hasEmptySlot(hopper) {
        const inv = HopperTools.getContainer(hopper);

        for (let i = 0; i < inv.size; i++)
            if (inv.getItem(i) === undefined)
                return true;

        return false;
    }

    static addNonStackableItem(hopper, itemName) {
        hopper.getComponent("inventory").container.addItem(new ItemStack(itemName));
    }
}

export { HopperTools };
